set.seed(1)
library(ape)
library(phytools)
source("as.phylo.formula.R")
library(Hmsc)

load(file = "processed_data/data_meta_otu.Rdata")
#meta,
#otu.table.plausible,otu.table.reliable,
#taxonomy.plausible, taxonomy.reliable,

prev.threshold = 50

ty = 1
if(ty==1){
  Y = otu.table.reliable
  taxonomy = taxonomy.reliable
} else {
  Y = otu.table.plausible
  taxonomy = taxonomy.plausible
}

all(rownames(taxonomy)==colnames(Y))
colnames(Y) = taxonomy$species

nall = dim(Y)[2]
print(paste0("total number of species-level OTUs in the full data: ",nall))

prev = colSums(Y>0)
sel.sp = prev>=prev.threshold

S.sel = rowSums(Y[,sel.sp]>0)
S.not.sel = rowSums(Y[,!sel.sp]>0)
Y.S = cbind(S.sel,S.not.sel)
Y = Y[,sel.sp]
taxonomy = taxonomy[sel.sp,]

sel.samples = meta$read.count>5000
median(meta$read.count[sel.samples])
min(meta$read.count[sel.samples])
max(meta$read.count[sel.samples])
Y = Y[sel.samples,]
meta = meta[sel.samples,]
Y.S = Y.S[sel.samples,]
dim(Y)

Y.pa = 1*(Y>0)
Y.abuc = Y
Y.abuc[Y==0] = NA
Y.abuc = log(Y.abuc)
Y.abuc = scale(Y.abuc)

#we include number of flowers and number of leaves (both modelled with log-normal Poisson for both cases). For individuals that were sampled in the second year, we included these variables two times in the response matrix, as they were measured for the first and the second year. For individuals that were sampled in the first year, we left the second-year measurements as missing data (NA). Note that we are mainly interested in the changes of these plant fitness variables from year one to year two. We evaluated these differences by the differences in the estimated regression parameters under the different treatments (1-12).
Bl.w.2018 = log(meta$`Bist_leafw 2018`) #normal
Bl.l.2018 = log(meta$`Bist_leafl 2018`) #normal
Bl.n.2018 = meta$`N_leafs 2018`#poisson
Bf.n.2018 = 1*(meta$`N_flowers 2018`>0) #probit
Bl.w.2019 = log(meta$`Bist_leafw 2019`) #normal
Bl.l.2019 = log(meta$`Bist_leafl 2019`) #normal
Bl.n.2019 = meta$`N_leafs 2019`#poisson
Bl.n.2019[Bl.n.2019==1.1]=1
Bl.n.2019 = round(Bl.n.2019)
Bf.n.2019 = 1*(meta$`N_flowers 2019`>0) #probit
table(Bl.n.2019)

Y.B = cbind(Bl.w.2018,Bl.w.2019,Bl.l.2018,Bl.l.2019,Bl.n.2018,Bl.n.2019,Bf.n.2018,Bf.n.2019)
plot(as.data.frame(Y.B))

translocated = 1*(meta$Treatment2 %in% c("Bhigh","Blow","Chigh","Clow","Dhigh","Dlow","Ehigh","Elow"))
lowtohigh = 1*(meta$Treatment2 %in% c("Clow","Elow"))
hightolow = 1*(meta$Treatment2 %in% c("Chigh","Ehigh"))
excluded = 1*(meta$Treatment2 %in% c("Bhigh","Blow","Chigh","Clow"))
altitude.of.origin = -(1/2) + (meta$Treatment2 %in% c("Ahigh","Bhigh","Chigh","Dhigh","Ehigh","Fhigh"))
altitude.of.sampling = -(1/2) + (meta$Treatment2 %in% c("Ahigh","Bhigh","Clow","Dhigh","Elow","Fhigh"))
turnover = -(1/2) + 1*(!meta$Treatment2 %in% c("Ahigh","Alow"))

tmp = list()

altitude = altitude.of.origin

turnover.altitude = turnover*altitude
translocated.altitude = translocated*altitude
excluded.altitude = excluded*altitude
excluded.lowtohigh = excluded*lowtohigh
excluded.hightolow = excluded*hightolow

ny = nrow(meta)
S.C = rep(NA,ny)
S.pH = rep(NA,ny)
for(i in 1:ny){
  sel = which(meta$Plot_id==meta$Bist_plot[i])
  S.C[i] = mean(meta$S.C[sel])
  S.pH[i] = mean(meta$S.pH[sel])
}

XData = data.frame(turnover,altitude,turnover.altitude,translocated,translocated.altitude,excluded,excluded.altitude,
                   lowtohigh,hightolow,excluded.lowtohigh,excluded.hightolow,sequencing.depth = scale(log(meta$read.count)),
                   S.C,S.pH)

XFormula.1 = ~ turnover + #C2
  altitude + #C3
  turnover.altitude + #C4
  translocated + #C5
  translocated.altitude + #C6
  excluded + #C7
  excluded.altitude + #C8
  lowtohigh + #C9
  hightolow + #C10
  excluded.lowtohigh + #C11
  excluded.hightolow + # C12
  sequencing.depth

XFormula.2 = ~ turnover + #C2
  altitude + #C3
  turnover.altitude + #C4
  translocated + #C5
  translocated.altitude + #C6
  excluded + #C7
  excluded.altitude + #C8
  lowtohigh + #C9
  hightolow + #C10
  excluded.lowtohigh + #C11
  excluded.hightolow + # C12
  sequencing.depth +
  S.C + 
  S.pH

studyDesign = droplevels(data.frame(id = as.factor(meta$`Bist_id 2018`),
                                    plot.of.sampling = as.factor(meta$Plot_id),
                                    plot.of.origin = as.factor(meta$Bist_plot),
                                    gradient=as.factor(meta$Gradient)))

rL.id = HmscRandomLevel(units = levels(studyDesign$id))
rL.plot.of.origin = HmscRandomLevel(units = levels(studyDesign$plot.of.origin))
rL.plot.of.sampling = HmscRandomLevel(units = levels(studyDesign$plot.of.sampling))
rL.gradient = HmscRandomLevel(units = levels(studyDesign$gradient))

my.tree = as.phylo.formula(~kingdom/phylum/class/order/family/genus/species,
                           data = taxonomy)

ma = match(my.tree$tip.label,colnames(Y.pa))
Y.pa = Y.pa[,ma]
Y.abuc = Y.abuc[,ma]
all(colnames(Y.pa)==my.tree$tip.label)
all(colnames(Y.abuc)==my.tree$tip.label)
plot(my.tree,cex=0.1)

m.pa = Hmsc(Y=Y.pa,
            distr="probit",
            XData = XData,  XFormula=XFormula.1,
            studyDesign = studyDesign, 
            ranLevels=list(id = rL.id, plot.of.sampling = rL.plot.of.sampling,plot.of.origin = rL.plot.of.origin, gradient = rL.gradient),
            phyloTree = my.tree)

m.pa.C.pH = Hmsc(Y=Y.pa,
              distr="probit",
              XData = XData,  XFormula=XFormula.2,
              studyDesign = studyDesign, 
              ranLevels=list(id = rL.id, plot.of.sampling = rL.plot.of.sampling,plot.of.origin = rL.plot.of.origin, gradient = rL.gradient),
              phyloTree = my.tree)

m.abuc = Hmsc(Y=Y.abuc,
              distr="normal",
              XData = XData,  XFormula=XFormula.1,
              studyDesign = studyDesign, 
              ranLevels=list(id = rL.id, plot.of.sampling = rL.plot.of.sampling,plot.of.origin = rL.plot.of.origin, gradient = rL.gradient),
              phyloTree = my.tree)

m.abuc.C.pH = Hmsc(Y=Y.abuc,
                distr="normal",
                XData = XData,  XFormula=XFormula.2,
                studyDesign = studyDesign, 
                ranLevels=list(id = rL.id, plot.of.sampling = rL.plot.of.sampling,plot.of.origin = rL.plot.of.origin, gradient = rL.gradient),
                phyloTree = my.tree)

m.S = Hmsc(Y=Y.S,
           distr="poisson",
           XData = XData,  XFormula=XFormula.1,
           studyDesign = studyDesign, 
           ranLevels=list(id = rL.id, plot.of.sampling = rL.plot.of.sampling,plot.of.origin = rL.plot.of.origin, gradient = rL.gradient))
           
m.S.C.pH = Hmsc(Y=Y.S,
             distr="poisson",
             XData = XData,  XFormula=XFormula.2,
             studyDesign = studyDesign, 
             ranLevels=list(id = rL.id, plot.of.sampling = rL.plot.of.sampling,plot.of.origin = rL.plot.of.origin, gradient = rL.gradient))

Y.all = cbind(Y.pa,Y.abuc,Y.B)
colnames(Y.all) = c(paste0(colnames(Y.pa),".pa"),paste0(colnames(Y.abuc),".abuc"),colnames(Y.B))

m.all = Hmsc(Y=Y.all,
             distr=c(rep("probit",ncol(Y.pa)),rep("normal",ncol(Y.abuc)),"normal","normal","normal","normal","poisson","poisson","probit","probit"),
             XData = XData,  XFormula=XFormula.1,
             studyDesign = studyDesign, 
             ranLevels=list(id = rL.id, plot.of.sampling = rL.plot.of.sampling,plot.of.origin = rL.plot.of.origin, gradient = rL.gradient))

m.all.C.pH = Hmsc(Y=Y.all,
               distr=c(rep("probit",ncol(Y.pa)),rep("normal",ncol(Y.abuc)),"normal","normal","normal","normal","poisson","poisson","probit","probit"),
               XData = XData,  XFormula=XFormula.2,
               studyDesign = studyDesign, 
               ranLevels=list(id = rL.id, plot.of.sampling = rL.plot.of.sampling,plot.of.origin = rL.plot.of.origin, gradient = rL.gradient))

load("models/unfitted_models_plant_fitness.RData")
m.Plant.fitness.model = models$Plant.fitness.model
m.Plant.fitness.model.C.pH = models$Plant.fitness.model.C.pH

models = list("Fungal.pa.model" = m.pa, "Fungal.pa.model.C.pH" = m.pa.C.pH,
              "Fungal.abuc.model" = m.abuc, "Fungal.abuc.model.C.pH" = m.abuc.C.pH,
              "Fungal.S.model" = m.S, "Fungal.S.model.C.pH" = m.S.C.pH,
              "Fungal.plant.model" = m.all, "Fungal.plant.model.C.pH" = m.all.C.pH,
              "m.Plant.fitness.model" = m.Plant.fitness.model,
              "m.Plant.fitness.model.C.pH" = m.Plant.fitness.model.C.pH)

save(models, file=paste0("models/unfitted_models_th_",as.character(prev.threshold),".RData"))


