##################################################################################################
# INPUT AND OUTPUT OF THIS SCRIPT:
##################################################################################################
#	INPUT. Unfitted models.

#	OUTPUT. Fitted models, with fitting done for multiple RUNs:
# first short MCMC chains (to provide some results fast), and then with increasingly long MCMC chains
# (until MCMC convergence or computational limit is reached): the files
# "models/models_thin_1_samples_5_chains_4.Rdata" (RUN 0),
# "models/models_thin_1_samples_250_chains_4.Rdata" (RUN 1),
# "models/models_thin_10_samples_250_chains_4.Rdata" (RUN 2), 
# "models/models_thin_100_samples_250_chains_4.Rdata" (RUN 3)
##################################################################################################

library(Hmsc)
prev.threshold = 50

load(file=paste0("models/unfitted_models_th_",as.character(prev.threshold),".RData"))
nm = length(models)
samples_list = c(5,250,250,250)
thin_list = c(1,1,10,100)
nChains = 4
nParallel = nChains
Lst = 1
while(Lst <= length(samples_list)){
  thin = thin_list[Lst]
  samples = samples_list[Lst]
  print(paste0("thin = ",as.character(thin),"; samples = ",as.character(samples)))
  filename = paste0("models/models_thin_", as.character(thin),
                                             "_samples_", as.character(samples),
                                             "_chains_",as.character(nChains),
                                             "_th_",as.character(prev.threshold),
                                             ".Rdata")
  if(file.exists(filename)){
    print("model had been fitted already")
  } else {
    print(date())
    print(filename)
    for (mi in 1:nm) {
      print(c(mi,nm))
      print(paste0("model = ",names(models)[mi]))
      m = models[[mi]]
      m = sampleMcmc(m, samples = samples, thin=thin,
                     adaptNf=rep(ceiling(0.4*samples*thin),m$nr), 
                     transient = ceiling(0.5*samples*thin),
                     nChains = nChains,
                     nParallel = nParallel) 
      models[[mi]] = m
    }
    save(models,file=filename)
  }
  Lst = Lst + 1
}